﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS {
  [Item("NumberOfSelectedSubScopesCalculator", "")]
  [StorableClass]
  public class NumberOfSelectedSubScopesCalculator : SingleSuccessorOperator {
    private ILookupParameter<IntValue> PopulationSizeParameter {
      get { return (ILookupParameter<IntValue>)Parameters["PopulationSize"]; }
    }
    private ILookupParameter<IntValue> ElitesParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Elites"]; }
    }
    private ILookupParameter<IntValue> NumberOfSelectedSubScopesParameter {
      get { return (ILookupParameter<IntValue>)Parameters["NumberOfSelectedSubScopes"]; }
    }

    [StorableConstructor]
    private NumberOfSelectedSubScopesCalculator(bool deserializing)
      : base(deserializing) { }
    private NumberOfSelectedSubScopesCalculator(NumberOfSelectedSubScopesCalculator original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NumberOfSelectedSubScopesCalculator(this, cloner);
    }
    public NumberOfSelectedSubScopesCalculator()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("PopulationSize"));
      Parameters.Add(new LookupParameter<IntValue>("Elites"));
      Parameters.Add(new LookupParameter<IntValue>("NumberOfSelectedSubScopes"));
    }

    public override IOperation Apply() {
      var populationSize = PopulationSizeParameter.ActualValue;
      var elites = ElitesParameter.ActualValue;

      int selectedSubScopes = 2 * (populationSize.Value - elites.Value);

      NumberOfSelectedSubScopesParameter.ActualValue = new IntValue(selectedSubScopes);

      return base.Apply();
    }
  }
}